@@ -8,7 +8,7 @@
 #'
 #' @param conc Vector of concentrations.
 #' @param resp Vector of responses.
-#' @param top Model top.
+#' @param top Model predicted top, maximal predicted change in response from baseline.
 #' @param cutoff Desired cutoff.
 #' @param er Model error parameter.
 #' @param ps Vector of used model parameters in order: a, tp, b, ga, p, la, q, er.
@@ -18,6 +18,9 @@
 #' @param errfun Which error distribution to assume for each point, defaults to
 #'   "dt4". "dt4" is the original 4 degrees of freedom t-distribution. Another
 #'   supported distribution is "dnorm", the normal distribution.
+#' @param poly2.biphasic Which fitting method to use for poly2. If poly2.biphasic = TRUE, allows for biphasic polynomial 2
+#'   model fits (i.e. both monotonic and non-monotonic). (Defaults to TRUE.)
+#' @param verbose If verbose = TRUE, will print status of empirical calculations. (Defaults to FALSE.)
 #'
 #' @importFrom stats pt
 #' @importFrom stats aggregate
@@ -38,7 +41,7 @@
 #' hitcontinner(conc,resp,top,cutoff = 0.8, er,ps,fit_method, caikwt, mll)
 #' hitcontinner(conc,resp,top,cutoff = 1, er,ps,fit_method, caikwt, mll)
 #' hitcontinner(conc,resp,top,cutoff = 1.2, er,ps,fit_method, caikwt, mll)
-hitcontinner = function(conc, resp, top, cutoff, er, ps, fit_method, caikwt, mll, errfun = "dt4"){
+hitcontinner = function(conc, resp, top, cutoff, er, ps, fit_method, caikwt, mll, errfun = "dt4", poly2.biphasic = TRUE, verbose = FALSE){
 
   #Each P represents the odds of the curve being a hit according to different criteria; multiply all Ps to get hit odds overall
   if(fit_method == "none") return(0)
@@ -64,7 +67,7 @@ hitcontinner = function(conc, resp, top, cutoff, er, ps, fit_method, caikwt, mll
   # P3 = pnorm((top-cutoff)/topsd) #odds of top above cutoff
   #assume ps may have nas in them
   ps = ps[!is.na(ps)]
-  P3 = toplikelihood(fname, cutoff, conc, resp, ps, top, mll, errfun = errfun) #odds of top above cutoff
+  P3 = toplikelihood(fname, cutoff, conc, resp, ps, top, mll, errfun = errfun, poly2.biphasic = poly2.biphasic, verbose = verbose) #odds of top above cutoff
 
   #multiply three probabilities
   return(P1*P2*P3)
